@@ -126,20 +126,38 @@ class BzrWikiMacros(core.Component):
             branches = repo.get_branches()
             rows = []
             for loc, target in branches:
-                revid = target.last_revision()
-                revno = target.revision_id_to_revno(revid)
-                rev = repo.string_rev(target, revid)
-                revision = target.repository.get_revision(revid)
-                timestamp = revision.timestamp
-                timestamp = dateinfo(timestamp)
-                rows.append(html.TR([html.TD(x, class_=c) for c, x in [
-                    ('name', html.A(loc, class_='dir',
-                                    href=req.href.browser(loc))),
-                    ('nick', target.nick),
-                    ('rev', html.A('[%d]'%revno, href=req.href.changeset(rev))),
-                    ('age', timestamp),
-                    ('change', revision.message),
-                    ]]))
+                try:
+                    revid = target.last_revision()
+                    revno = target.revision_id_to_revno(revid)
+                    rev = repo.string_rev(target, revid)
+                    if revid == NULL_REVISION:
+                        timestamp = u'\u2014'
+                        message = 'This branch is empty'
+                    else:
+                        revision = target.repository.get_revision(revid)
+                        timestamp = revision.timestamp
+                        timestamp = dateinfo(timestamp)
+                        message = revision.message
+                    rows.append(html.TR([html.TD(x, class_=c) for c, x in [
+                        ('name', html.A(loc, class_='dir',
+                                        href=req.href.browser(loc))),
+                        ('nick', target.nick),
+                        ('rev', html.A('[%d]'%revno,
+                                       href=req.href.changeset(rev))),
+                        ('age', timestamp),
+                        ('change', message),
+                        ]]))
+                except errors.BzrError, e:
+                    # In case of an unexpected error, don't die completely
+                    rows.append(html.TR(
+                        html.TD(html.A(loc, class_='dir',
+                                       href=req.href.browser(loc)),
+                                class_='name'),
+                        html.TD(html.DIV(
+                            html.STRONG('Error getting branch details'),
+                            html.PRE(str(e)),
+                            class_='system-message'),
+                                colspan='4')))
             return html.TABLE(class_='listing')(
                 html.THEAD(html.TR([html.TH(x) for x in [
                     'Path', 'Nick', 'Revision', 'Age', 'Last Change'
@@ -538,12 +556,10 @@ class BzrRepository(versioncontrol.Repository):
         return klass(self, branch, tree, entry, path)
 
     def get_oldest_rev(self):
-        if self._history is None:
-            self._history = self._repo_history()
-        if self._history:
-            return self.string_rev(self._history[0][1], self._history[0][0])
-        else:
-            return self.string_rev(None, NULL_REVISION)
+        # Retrieving the oldest real revision in the entire repository is
+        # *extremely* slow.
+        # By definition, this revision is always in the branch history.
+        return self.string_rev(None, NULL_REVISION)
 
     def get_youngest_rev(self):
         # XXX: Find a better solution
